package com.Komunaldata.SmsGateway.Response;

import java.util.Date;

/**
 * SMSOut class for storing outgoing SMSes retrieved by calling getSmsOutBy[DedicatedAccount/NonDedicatedAccount/User] webservice.
 * @author Developer7
 */
public class SMSOut {

    private wapInfoClass wapInfo;
    private String[] selectedModemsToSend;
    private Boolean concatenated;
    private String wantedCom;
    private String recipientProvider;
    private String Text;
    private Integer id;
    private Date tryingDate;
    private Boolean repeatedSending;
    private String coding;
    private Boolean flash;
    private Boolean ringTone;
    private Boolean wapPush;
    private Boolean dedicated;
    private Date wantedSendingTime;
    private String usedSenderPhoneNumber;
    private String senderPhoneNumber;
    private String recipientPhoneNumber;
    private Short part_count;
    private Integer account;
    private String user;
    private String status;
    private Date smsSendedTime;
    private Date smsStatusReportRecieved;

    /**
     * 
     * @param wapInfo WapInfoClass type data that holds information about the message if it is an wap push message
     * @param selectedModemsToSend Selected modems to send the sms message
     * @param concatenated The sms message is concatenated
     * @param wantedCom OBSOLETE?
     * @param recipientProvider Phone provider of the recipient ( like in Hungary "20" as tenenor,"30" as T-Com,"70" as Vodafone )
     * @param Text Message of the normal sms message (noral means not wap message)
     * @param id Identity of the sms
     * @param tryingDate Date and time of the desired sending date. The system won't send out the sms before this Date-time
     * @param repeatedSending This message has more than one recipient so it is an multitudinous sms
     * @param coding Can be "7" or "16". SMS coding can be 7bit for GSM characters or 16bit for Unicode, one SMS can hold 160 7bit chars or 70 16bit chars, for sending longer messages the system uses concatenated SMS-es
     * @param flash This message is an Flash sms message
     * @param ringTone This message is an ringtone
     * @param wapPush This message is an wap push message
     * @param dedicated Means the selectedModemsToSend parameter has modems. The message will be sent from one of these modems
     * @param wantedSendingTime OBSOLETE?
     * @param usedSenderPhoneNumber The message was sent from this number
     * @param senderPhoneNumber OBSOLETE?
     * @param recipientPhoneNumber Phone number of the recipient
     * @param part_count How many concat-parts has the message
     * @param account Account of the sender
     * @param user Sender user
     * @param status Status of the message for example "Feldolgozatlan" means the sms is not loaded to the system application yet
     * @param smsSendedTime The date when the message was sent out
     * @param smsStatusReportRecieved The user obtained the message
     */
    public SMSOut(wapInfoClass wapInfo, String[] selectedModemsToSend, Boolean concatenated, String wantedCom, String recipientProvider, String Text, Integer id, Date tryingDate, Boolean repeatedSending, String coding, Boolean flash, Boolean ringTone, Boolean wapPush, Boolean dedicated, Date wantedSendingTime, String usedSenderPhoneNumber, String senderPhoneNumber, String recipientPhoneNumber, Short part_count, Integer account, String user, String status, Date smsSendedTime, Date smsStatusReportRecieved)
    {
        this.wapInfo = wapInfo;
        this.selectedModemsToSend = selectedModemsToSend;
        this.concatenated = concatenated;
        this.wantedCom = wantedCom;
        this.recipientProvider = recipientProvider;
        this.Text = Text;
        this.id = id;
        this.tryingDate = tryingDate;
        this.repeatedSending = repeatedSending;
        this.coding = coding;
        this.flash = flash;
        this.ringTone = ringTone;
        this.wapPush = wapPush;
        this.dedicated = dedicated;
        this.wantedSendingTime = wantedSendingTime;
        this.usedSenderPhoneNumber = usedSenderPhoneNumber;
        this.senderPhoneNumber = senderPhoneNumber;
        this.recipientPhoneNumber = recipientPhoneNumber;
        this.part_count = part_count;
        this.account = account;
        this.user = user;
        this.status = status;
        this.smsSendedTime = smsSendedTime;
        this.smsStatusReportRecieved = smsStatusReportRecieved;
    }

    /**
     *
     * @return
     */
    public String getText()
    {
	return Text;
    }

    /**
     *
     * @return
     */
    public int getAccount()
    {
	return account;
    }

    /**
     *
     * @return
     */
    public String getCoding()
    {
	return coding;
    }

    /**
     *
     * @return
     */
    public boolean isConcatenated()
    {
	return concatenated;
    }

    /**
     *
     * @return
     */
    public boolean isDedicated()
    {
	return dedicated;
    }

    /**
     *
     * @return
     */
    public boolean isFlash()
    {
	return flash;
    }

    /**
     *
     * @return
     */
    public int getId()
    {
	return id;
    }

    /**
     *
     * @return
     */
    public short getPart_count()
    {
	return part_count;
    }

    /**
     *
     * @return
     */
    public String getRecipientPhoneNumber()
    {
	return recipientPhoneNumber;
    }

    /**
     *
     * @return
     */
    public String getRecipientProvider()
    {
	return recipientProvider;
    }

    /**
     *
     * @return
     */
    public boolean isRepeatedSending()
    {
	return repeatedSending;
    }

    /**
     *
     * @return
     */
    public boolean isRingTone()
    {
	return ringTone;
    }

    /**
     *
     * @return
     */
    public String[] getSelectedModemsToSend()
    {
	return selectedModemsToSend;
    }

    /**
     *
     * @return
     */
    public String getSenderPhoneNumber()
    {
	return senderPhoneNumber;
    }

    /**
     *
     * @return
     */
    public Date getSmsSendedTime()
    {
	return smsSendedTime;
    }

    /**
     *
     * @return
     */
    public Date getSmsStatusReportRecieved()
    {
	return smsStatusReportRecieved;
    }

    /**
     *
     * @return
     */
    public String getStatus()
    {
	return status;
    }

    /**
     *
     * @return
     */
    public Date getTryingDate()
    {
	return tryingDate;
    }

    /**
     *
     * @return
     */
    public String getUsedSenderPhoneNumber()
    {
	return usedSenderPhoneNumber;
    }

    /**
     *
     * @return
     */
    public String getUser()
    {
	return user;
    }

    /**
     *
     * @return
     */
    public String getWantedCom()
    {
	return wantedCom;
    }

    /**
     *
     * @return
     */
    public Date getWantedSendingTime()
    {
	return wantedSendingTime;
    }

    /**
     * 
     * @return
     */
    public wapInfoClass getWapInfo()
    {
	return wapInfo;
    }

    /**
     *
     * @return
     */
    public boolean isWapPush()
    {
	return wapPush;
    }

    

}
