/**
 * Calling webservices for sending SMS.
 *
 */
package com.Komunaldata.SmsGateway;
import com.Komunaldata.SmsGateway.Response.*;
import java.io.IOException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
/**
 * <pre>
 * This class is created for calling webservices provided by Komunáldata Kft.
 *
 * Definitions:
 *	*SMSUser: Individuals with username/password.
 *	*Account: Individual users belong to an account, one user can only belong to one given account.
 *	*Credit: Credit is used for sending SMS-es one credit is valid for one SMS (one message can use more than one credit according to it's length and coding, for proper credit usage use getSMSLenght() function prior sending a SMS).
 *	*Modem: The unit for sending and recieving SMS-es. Each modem is identified by the phone number of its sim card.
 *		    *Dedicated Modem: Dedicated modems can be used only from the account it is dedicated to.
 *		    *NonDedicated Modem: Modems which can be used with multiple accounts.
 *	*Coding: SMS coding can be 7bit for GSM characters or 16bit for Unicode, one SMS can hold 160 7bit chars or 70 16bit chars, for sending longer messages the system uses concatenated SMS-es.
 *	*Concatenates SMS: A long SMS which can be sent in several parts (upto 256 piece according to the GSM standard) which will arrive as one SMS to the recipient.
 * 
 *</pre>
 * @author Developer7
 */
public class SmsGateway
{
    private static final String serverUrl = "http://govcenter.hu/smsgateway/service1.asmx";
    /**
     * Deleting a single incoming SMS by its ID.
     * @param userName Username of the user.
     * @param password Password of the user.
     * @param smsId The id of the SMS for deletion.
     * @return	deleteSmsResponse
     * @throws IOException
     */
    public static deleteSmsResponse deleteSmsInById(String userName, String password, Integer smsId) throws IOException
    {
	return deleteSmsInById(new SMSUser(userName, password), smsId);
    }
    /**
     * Deleting a single incoming SMS by its ID.
     * @param user SMSUser (username,password).
     * @param smsId The id of the SMS for deletion.
     * @return deleteSmsResponse
     * @throws IOException
     */
    public static deleteSmsResponse deleteSmsInById(SMSUser user, Integer smsId) throws IOException
    {
	Integer[] passValue = new Integer[1];
	passValue[0] = smsId;
	return deleteSmsInById(user, passValue);
    }
    /**
     * Deleting multiple incoming SMS-es by their ID.
     * @param userName Username of the user.
     * @param password Password of the user.
     * @param smsIdList List of id of SMS-es for deletion.
     * @return deleteSmsResponse
     * @throws IOException
     */
    public static deleteSmsResponse deleteSmsInById(String userName, String password, Integer[] smsIdList) throws IOException
    {
	return deleteSmsInById(new SMSUser(userName, password), smsIdList);
    }
    /**
     * Deleting multiple incoming SMS-es by their ID.
     * @param user SMSUser (username,password).
     * @param smsIdList List of id of SMS-es for deletion.
     * @return deleteSmsResponse
     * @throws IOException
     */
    public static deleteSmsResponse deleteSmsInById(SMSUser user, Integer[] smsIdList) throws IOException
    {
	String webService = "deleteSmsInByIdList";

	String content = getContentHeader();
	content += getWebServiceHeader(webService);

	content += getUserToString(user);

	//
	content += "<idList>\r\n";
	for (int i = 0; i < smsIdList.length; i++) {
	    content += "<int>" + smsIdList[i] + "</int>\r\n";
	}
	content += "</idList>\r\n";
	//

	content += getWebServiceEnd(webService);
	content += getContentEnd();
	Document response = null;
	try {
	    response = sendData(content, webService);
	} catch (ParserConfigurationException ex) {
	    Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
	} catch (SAXException ex) {
	    Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
	}

	deleteSmsResponse ret = pharseDeleteSmsResponse(response);

	return ret;
    }
    /**
     * Deleting a single outgoing SMS by its ID.
     * @param userName Username of the user.
     * @param password Password of the user.
     * @param smsId The id of the SMS for deletion.
     * @return deleteSmsResponse
     * @throws IOException
     */
    public static deleteSmsResponse deleteSmsOutById(String userName, String password, Integer smsId) throws IOException
    {
	return deleteSmsOutById(new SMSUser(userName, password), smsId);
    }
    /**
     * Deleting a single outgoing SMS by its ID.
     * @param user SMSUser (username,password).
     * @param smsId The id of the SMS for deletion.
     * @return deleteSmsResponse
     * @throws IOException
     */
    public static deleteSmsResponse deleteSmsOutById(SMSUser user, Integer smsId) throws IOException
    {
	Integer[] passValue = new Integer[1];
	passValue[0] = smsId;
	return deleteSmsOutById(user, passValue);
    }
    /**
     * Deleting multiple outgoing SMS by its ID.
     * @param userName Username of the user.
     * @param password Password of the user.
     * @param smsIdList List of id of SMS-es for deletion.
     * @return deleteSmsResponse
     * @throws IOException
     */
    public static deleteSmsResponse deleteSmsOutById(String userName, String password, Integer[] smsIdList) throws IOException
    {
	return deleteSmsOutById(new SMSUser(userName, password), smsIdList);
    }
    /**
     * Deleting multiple outgoing SMS by its ID.
     * @param user SMSUser (username,password).
     * @param smsIdList List of id of SMS-es for deletion.
     * @return deleteSmsResponse
     * @throws IOException
     */
    public static deleteSmsResponse deleteSmsOutById(SMSUser user, Integer[] smsIdList) throws IOException
    {
	String webService = "deleteSmsOutByIdList";

	String content = getContentHeader();
	content += getWebServiceHeader(webService);

	content += getUserToString(user);

	//
	content += "<idList>\r\n";
	for (Integer i = 0; i < smsIdList.length; i++) {
	    content += "<Integer>" + smsIdList[i] + "</Integer>\r\n";
	}
	content += "</idList>\r\n";
	//

	content += getWebServiceEnd(webService);
	content += getContentEnd();
	Document response = null;
	try {
	    response = sendData(content, webService);
	} catch (ParserConfigurationException ex) {
	    Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
	} catch (SAXException ex) {
	    Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
	}

	deleteSmsResponse ret = pharseDeleteSmsResponse(response);

	return ret;
    }
    /**
     * Gets the credits available for a given account.
     * @param userName Username of the user.
     * @param password Password of the user.
     * @param accountId The Id of the account.
     * @return	getCreditResponse: Integer returnValue
     * @throws IOException
     */
    public static getCreditResponse getCreditAccount(String userName, String password, short accountId) throws IOException
    {
	return getCreditAccount(new SMSUser(userName, password), accountId);
    }
    /**
     * Gets the credits available for a given account.
     * @param user SMSUser (username,password).
     * @param accountId The Id of the account.
     * @return getCreditResponse: Integer returnValue
     * @throws IOException
     */
    public static getCreditResponse getCreditAccount(SMSUser user, short accountId) throws IOException
    {
	String webService = "getCreditAccount";

	String content = getContentHeader();
	content += getWebServiceHeader(webService);
	//
	content += "<accountId>" + accountId + "</accountId>\r\n";
	//
	content += getUserToString(user);
	content += getWebServiceEnd(webService);
	content += getContentEnd();
	Document response = null;
	try {
	    response = sendData(content, webService);
	} catch (ParserConfigurationException ex) {
	    Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
	} catch (SAXException ex) {
	    Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
	}

	getCreditResponse ret = pharseGetCreditResponse(response);

	return ret;
    }
    /**
     * Gets the credits available for the user.
     * @param userName Username of the user.
     * @param password Password of the user.
     * @return getCreditResponse: Integer returnValue
     * @throws IOException
     */
    public static getCreditResponse getCreditUser(String userName, String password) throws IOException
    {
	return getCreditUser(new SMSUser(userName, password));
    }
    /**
     * Gets the credits available for the user.
     * @param user SMSUser (username,password).
     * @return getCreditResponse: Integer returnValue
     * @throws IOException
     */
    public static getCreditResponse getCreditUser(SMSUser user) throws IOException
    {

	String webService = "getCreditUser";

	String content = getContentHeader();
	content += getWebServiceHeader(webService);

	content += getUserToString(user);

	content += getWebServiceEnd(webService);
	content += getContentEnd();
	Document response = null;
	try {
	    response = sendData(content, webService);
	} catch (ParserConfigurationException ex) {
	    Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
	} catch (SAXException ex) {
	    Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
	}

	getCreditResponse ret = pharseGetCreditResponse(response);

	return ret;
    }
    /**
     * Gets the dedicated modems belonging to a given account.<br>
     * Returns a list of string containing the phone number of the dedicated modems if any.
     * @param userName Username of the user.
     * @param password Password of the user.
     * @param accountId The Id of the account.
     * @return getModemsByAccountResponse: String[] returnValue
     * @throws IOException
     */
    public static getModemsByAccountResponse getModemsByAccount(String userName, String password, short accountId) throws IOException
    {
	return getModemsByAccount(new SMSUser(userName, password), accountId);
    }
    /**
     * Gets the dedicated modems belonging to a given account.<br>
     * Returns a list of string containing the phone number of the dedicated modems if any.
     * @param user SMSUser (username,password).
     * @param accountId The Id of the account.
     * @return getModemsByAccountResponse: String[] returnValue
     * @throws IOException
     */
    public static getModemsByAccountResponse getModemsByAccount(SMSUser user, short accountId) throws IOException
    {
	String webService = "getModemsByAccount";

	String content = getContentHeader();
	content += getWebServiceHeader(webService);
	//
	content += "<accountId>" + accountId + "</accountId>\r\n";
	//
	content += getUserToString(user);
	content += getWebServiceEnd(webService);
	content += getContentEnd();
	Document response = null;
	try {
	    response = sendData(content, webService);
	} catch (ParserConfigurationException ex) {
	    Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
	} catch (SAXException ex) {
	    Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
	}

	getModemsByAccountResponse ret = pharseGetModemsByAccountResponse(
		response);

	return ret;
    }
    /**
     * Gets the exact length of the SMS.<br>
     *	    * 7bit SMS can hold 160 GSM chars<br>
     *	    * 16bit SMS can hold 70 Unicode chars<br>
     * Longer messages will be concatenated and sent in different parts (upto 256 parts) and delivered as one message to the recipient.<br>
     * This functions tells if the SMS has to be concatenated or not and the number of parts it is concatenated to.<br>
     * Credit usage of the SMS is calculated with this function, every part of the SMS uses one credit.<br>
     * 
     * @param coding "7" for 7bit, "16" for 16bit - 8bit is not implemented.
     * @param message The text of the SMS.
     * @return getSMSLength: Integer returnValue
     * @throws IOException
     */
    public static getSmsLengthResponse getSMSLength(String coding, String message) throws IOException
    {
	return getSMSLength(coding, message, Boolean.FALSE, null, null, null, null, null, null, null);
    }
    /**
     * Gets the exact length of the SMS.<br>
     *	    * 7bit SMS can hold 160 GSM chars<br>
     *	    * 16bit SMS can hold 70 Unicode chars<br>
     * Longer messages will be concatenated and sent in different parts (upto 256 parts) and delivered as one message to the recipient.<br>
     * This functions tells if the SMS has to be concatenated or not and the number of parts it is concatenated to.<br>
     * Credit usage of the SMS is calculated with this function, every part of the SMS uses one credit.<br>
     * 
     * @param coding "7" for 7bit, "16" for 16bit - 8bit is not implemented.
     * @param message The text of the SMS.
     * @param wap
     * @param wlink
     * @param wmessage
     * @param mode
     * @param href
     * @param type
     * @param validityTime
     * @param releaseTime
     * @return getSMSLength: Integer returnValue
     * @throws IOException
     */
    public static getSmsLengthResponse getSMSLength(String coding, String message, Boolean wap, Boolean wlink, Boolean wmessage, String mode, String href, String type, Date validityTime, Date releaseTime) throws IOException
    {
	String webService = "getSMSLength";

	String content = getContentHeader();
	content += getWebServiceHeader(webService);
	//
	if (coding != null) {
	    content += "<coding>" + coding + "</coding>\r\n";
	}
	if (message != null) {
	    content += "<message>" + message + "</message>\r\n";
	}
	if (wap != null) {
	    content += "<wap>" + wap + "</wap>\r\n";
	}
	if (wlink != null) {
	    content += "<wlink>" + wlink + "</wlink>\r\n";
	}
	if (wmessage != null) {
	    content += "<wmessage>" + wmessage + "</wmessage>\r\n";
	}
	if (mode != null) {
	    content += "<mode>" + mode + "</mode>\r\n";
	}
	if (href != null) {
	    content += "<href>" + href + "</href>\r\n";
	}
	if (validityTime != null) {
	    DateFormat df = new SimpleDateFormat("yyyy-MM-dd-HH:mm:ss.SSS");
	    String sendingdate = df.format(validityTime);
	    char[] data = sendingdate.toCharArray();
	    data[10] = 'T';
	    sendingdate = new String(data);
	    content += "<validityTime>" + validityTime + "</validityTime>\r\n";
	}
	if (releaseTime != null) {
	    DateFormat df = new SimpleDateFormat("yyyy-MM-dd-HH:mm:ss.SSS");
	    String sendingdate = df.format(releaseTime);
	    char[] data = sendingdate.toCharArray();
	    data[10] = 'T';
	    sendingdate = new String(data);
	    content += "<releaseTime>" + releaseTime + "</releaseTime>\r\n";
	}
	//
	content += getWebServiceEnd(webService);
	content += getContentEnd();
	Document response = null;
	try {
	    response = sendData(content, webService);
	} catch (ParserConfigurationException ex) {
	    Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
	} catch (SAXException ex) {
	    Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
	}

	getSmsLengthResponse ret = pharseGetSmsLengthResponse(response);
	return ret;
    }
    /**
     * Gets all incoming SMS which is recieved by dedicated modem for the account the user is member of.
     * @param userName Username of the user.
     * @param password Password of the user.
     * @return getSmsInResponse: SMSIn[] returnValue
     * @throws IOException
     */
    public static getSmsInResponse getSmsInByDedicatedAccount(String userName, String password) throws IOException
    {
	return getSmsInByDedicatedAccount(new SMSUser(userName, password));
    }
    /**
     * Gets all incoming SMS which is recieved by dedicated modem for the account the user is member of.
     * @param user SMSUser (username,password).
     * @return getSmsInResponse: SMSIn[] returnValue
     * @throws IOException
     */
    public static getSmsInResponse getSmsInByDedicatedAccount(SMSUser user) throws IOException
    {
	String webService = "getSmsInByDedicatedAccount";

	String content = getContentHeader();
	content += getWebServiceHeader(webService);
	//

	//
	content += getUserToString(user);
	content += getWebServiceEnd(webService);
	content += getContentEnd();
	Document response = null;
	try {
	    response = sendData(content, webService);
	} catch (ParserConfigurationException ex) {
	    Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
	} catch (SAXException ex) {
	    Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
	}

	getSmsInResponse ret = parseGetSmsInResponse(response);

	return ret;
    }
    /**
     * Gets all incoming SMS which is recieved by dedicated modem for the account the user is member of.
     * @param userName Username of the user.
     * @param password Password of the user.
     * @return getSmsInResponse: SMSIn[] returnValue
     * @throws IOException
     */
    public static getSmsInResponse getSmsInByNonDedicatedAccount(String userName, String password) throws IOException
    {
	return getSmsInByNonDedicatedAccount(new SMSUser(userName, password));
    }
    /**
     * Gets all incoming SMS which is recieved by non-dedicated modem for the account the user is member of.
     * @param user SMSUser (username,password).
     * @return getSmsInResponse: SMSIn[] returnValue
     * @throws IOException
     */
    public static getSmsInResponse getSmsInByNonDedicatedAccount(SMSUser user) throws IOException
    {
	String webService = "getSmsInByNonDedicatedAccount";

	String content = getContentHeader();
	content += getWebServiceHeader(webService);
	//

	//
	content += getUserToString(user);
	content += getWebServiceEnd(webService);
	content += getContentEnd();
	Document response = null;
	try {
	    response = sendData(content, webService);
	} catch (ParserConfigurationException ex) {
	    Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
	} catch (SAXException ex) {
	    Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
	}

	getSmsInResponse ret = parseGetSmsInResponse(response);

	return ret;
    }
    /**
     * Gets all incoming SMS for the user.
     * @param userName Username of the user.
     * @param password Password of the user.
     * @return getSmsInResponse: SMSIn[] returnValue
     * @throws IOException
     */
    public static getSmsInResponse getSmsInByUser(String userName, String password) throws IOException
    {
	return getSmsInByUser(new SMSUser(userName, password));
    }
    /**
     * Gets all incoming SMS for the user.
     * @param user SMSUser (username,password).
     * @return getSmsInResponse: SMSIn[] returnValue
     * @throws IOException
     */
    public static getSmsInResponse getSmsInByUser(SMSUser user) throws IOException
    {
	String webService = "getSmsInByUser";

	String content = getContentHeader();
	content += getWebServiceHeader(webService);
	//

	//
	content += getUserToString(user);
	content += getWebServiceEnd(webService);
	content += getContentEnd();
	Document response = null;
	try {
	    response = sendData(content, webService);
	} catch (ParserConfigurationException ex) {
	    Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
	} catch (SAXException ex) {
	    Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
	}

	getSmsInResponse ret = parseGetSmsInResponse(response);

	return ret;
    }
    /**
     * Gets all outgoing SMS for the account which the user is member of.
     * @param userName Username of the user.
     * @param password Password of the user.
     * @param listDeleted If set to true, it will show the SMS-es marked for deletion.
     * @return getSmsOutResponse: SMSOut[] returnValue
     * @throws IOException
     */
    public static getSmsOutResponse getSmsOutByAccount(String userName, String password, boolean listDeleted) throws IOException
    {
	return getSmsOutByAccount(new SMSUser(userName, password), listDeleted);
    }
    /**
     * Gets all outgoing SMS for the account which the user is member of.
     * @param user SMSUser (username,password).
     * @param listDeleted If set to true, it will show the SMS-es marked for deletion.
     * @return getSmsOutResponse: SMSOut[] returnValue
     * @throws IOException
     */
    public static getSmsOutResponse getSmsOutByAccount(SMSUser user, boolean listDeleted) throws IOException
    {
	String webService = "getSmsOutByAccount";

	String content = getContentHeader();
	content += getWebServiceHeader(webService);

	//
	content += getUserToString(user);
	content += "<listDeleted>" + listDeleted + "</listDeleted>\r\n";
	//
	content += getWebServiceEnd(webService);
	content += getContentEnd();
	Document response = null;
	try {
	    response = sendData(content, webService);
	} catch (ParserConfigurationException ex) {
	    Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
	} catch (SAXException ex) {
	    Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
	}

	getSmsOutResponse ret = parseGetSmsOutResponse(response);

	return ret;
    }
    /**
     * Gets a single outgoing SMS by its Id.
     * @param userName Username of the user.
     * @param password Password of the user.
     * @param sms_id The Id of the SMS.
     * @return getSmsOutResponse: SMSOut[] returnValue
     * @throws IOException
     */
    public static getSmsOutResponse getSmsOutById(String userName, String password, Integer sms_id) throws IOException
    {
	return getSmsOutById(new SMSUser(userName, password), sms_id);
    }
    /**
     * Gets a single outgoing SMS by its Id.
     * @param user SMSUser (username,password).
     * @param sms_id The Id of the SMS.
     * @return getSmsOutResponse: SMSOut[] returnValue
     * @throws IOException
     */
    public static getSmsOutResponse getSmsOutById(SMSUser user, Integer sms_id) throws IOException
    {
	String webService = "getSmsOutById";

	String content = getContentHeader();
	content += getWebServiceHeader(webService);

	//
	content += getUserToString(user);
	content += "<sms_id>" + sms_id + "</sms_id>\r\n";
	//
	content += getWebServiceEnd(webService);
	content += getContentEnd();
	Document response = null;
	try {
	    response = sendData(content, webService);
	} catch (ParserConfigurationException ex) {
	    Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
	} catch (SAXException ex) {
	    Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
	}

	getSmsOutResponse ret = parseGetSmsOutResponseId(response);

	return ret;

    }
    /**
     * Gets all outgoing SMS for the user.
     * @param userName Username of the user.
     * @param password Password of the user.
     * @param listDeleted If set to true, it will show the SMS-es marked for deletion.
     * @return getSmsOutResponse: SMSOut[] returnValue
     * @throws IOException
     */
    public static getSmsOutResponse getSmsOutByUser(String userName, String password, boolean listDeleted) throws IOException
    {
	return getSmsOutByUser(new SMSUser(userName, password), listDeleted);
    }
    /**
     * Gets all outgoing SMS for the user.
     * @param user SMSUser (username,password).
     * @param listDeleted If set to true, it will show the SMS-es marked for deletion.
     * @return getSmsOutResponse: SMSOut[] returnValue
     * @throws IOException
     */
    public static getSmsOutResponse getSmsOutByUser(SMSUser user, boolean listDeleted) throws IOException
    {
	String webService = "getSmsOutByUser";

	String content = getContentHeader();
	content += getWebServiceHeader(webService);

	//
	content += getUserToString(user);
	content += "<listDeleted>" + listDeleted + "</listDeleted>\r\n";
	//
	content += getWebServiceEnd(webService);
	content += getContentEnd();
	Document response = null;
	try {
	    response = sendData(content, webService);
	} catch (ParserConfigurationException ex) {
	    Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
	} catch (SAXException ex) {
	    Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
	}

	getSmsOutResponse ret = parseGetSmsOutResponse(response);

	return ret;
    }
    /**
     * Send SMS.
     * @param userName Username of the user.
     * @param password Password of the user.
     * @param text The text of the SMS.
     * @param bit Coding "7" for 7bit and "16" for 16bit codeing.
     * @param recipientList List of string containing the phone number of recipients.
     * @param wantedSenderPhones List of dedicated modems to use for sending the SMS.
     * @return insertSmsOutNormal: Integer[] retrunValue - the ID of the SMS
     * @throws IOException
     */
    public static insertSmsResponse insertSmsOutNormal(String userName, String password, String text, String bit, String[] recipientList, String[] wantedSenderPhones) throws IOException
    {
	return insertSmsOutNormal(new SMSUser(userName, password), text, bit, recipientList, wantedSenderPhones);
    }
    /**
     * Send SMS.
     * @param user SMSUser (username,password).
     * @param text The text of the SMS.
     * @param bit Coding "7" for 7bit and "16" for 16bit codeing.
     * @param recipientList List of string containing the phone number of recipients.
     * @param wantedSenderPhones List of dedicated modems to use for sending the SMS.
     * @return insertSmsOutNormal: Integer[] retrunValue - the ID of the SMS
     * @throws IOException
     */
    public static insertSmsResponse insertSmsOutNormal(SMSUser user, String text, String bit, String[] recipientList, String[] wantedSenderPhones) throws IOException
    {
	String webService = "insertSmsOutNormal";

	String content = getContentHeader();
	content += getWebServiceHeader(webService);

	//
	content += "<text>" + text + "</text>\r\n";
	content += "<bit>" + bit + "</bit>\r\n";

	content += "<recipientList>\r\n";
	for (Integer i = 0; i < recipientList.length; i++) {
	    content += "<string>" + recipientList[i] + "</string>\r\n";
	}
	content += "</recipientList>\r\n";

	
	if (wantedSenderPhones != null) {
	    content += "<wantedSenderPhones>\r\n";
	    for (Integer i = 0; i < wantedSenderPhones.length; i++) {
		if (wantedSenderPhones[i] != null) {
		    content += "<string>" + wantedSenderPhones[i] + "</string>";
		}
	    }
	    content += "</wantedSenderPhones>\r\n";
	} else {
	    content += "<wantedSenderPhones />\r\n";
	}
	
	//
	content += getUserToString(user);

	content += getWebServiceEnd(webService);
	content += getContentEnd();
	Document response = null;
	try {
	    response = sendData(content, webService);
	} catch (ParserConfigurationException ex) {
	    Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
	} catch (SAXException ex) {
	    Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
	}

	insertSmsResponse ret = parseInsertSmsResponse(response);

	return ret;

    }
    /**
     * Send 7bit coded SMS with timing.
     * @param userName Username of the user.
     * @param password Password of the user.
     * @param text The text of the SMS.
     * @param recipientList List of string containing the phone number of recipients.
     * @param wantedSenderPhones List of dedicated modems to use for sending the SMS.
     * @param sendingDate The date when the SMS should be sent.
     * @return insertSmsOutNormal: Integer[] retrunValue - the ID of the SMS
     * @throws IOException
     */
    public static insertSmsResponse insertSmsOutNormal7BitWithTiming(String userName, String password, String text, String[] recipientList, String[] wantedSenderPhones, java.util.Date sendingDate) throws IOException
    {
	return insertSmsOutNormal7BitWithTiming(new SMSUser(userName, password), text, recipientList, wantedSenderPhones, sendingDate);
    }
    /**
     * Send 7bit coded SMS with timing.
     * @param user SMSUser (username,password).
     * @param text The text of the SMS.
     * @param recipientList List of string containing the phone number of recipients.
     * @param wantedSenderPhones List of dedicated modems to use for sending the SMS.
     * @param sendingDate The date when the SMS should be sent.
     * @return insertSmsOutNormal: Integer[] retrunValue - the ID of the SMS
     * @throws IOException
     */
    public static insertSmsResponse insertSmsOutNormal7BitWithTiming(SMSUser user, String text, String[] recipientList, String[] wantedSenderPhones, java.util.Date sendingDate) throws IOException
    {
	String webService = "insertSmsOutNormal7BitWithTiming";

	String content = getContentHeader();
	content += getWebServiceHeader(webService);

	//
	content += "<text>" + text + "</text>\r\n";

	DateFormat df = new SimpleDateFormat("yyyy-MM-dd-HH:mm:ss.SSS");
	String sendingdate = df.format(sendingDate);
	char[] data = sendingdate.toCharArray();
	data[10] = 'T';
	sendingdate = new String(data);
	content += "<sendingdate>" + sendingdate + "</sendingdate>\r\n";

	content += "<recipientList>\r\n";
	for (Integer i = 0; i < recipientList.length; i++) {
	    content += "<string>" + recipientList[i] + "</string>\r\n";
	}
	content += "</recipientList>\r\n";

	if (wantedSenderPhones != null) {
	    for (Integer i = 0; i < wantedSenderPhones.length; i++) {
		if (i == 0) {
		    content += "<wantedSenderPhones>\r\n";
		}
		if (wantedSenderPhones[i] != null) {
		    content += "<user>" + wantedSenderPhones[i] + "</user>";
		}
		if (i == wantedSenderPhones.length - 1) {
		    content += "</wantedSenderPhones>\r\n";
		}
	    }
	}

	//
	content += getUserToString(user);

	content += getWebServiceEnd(webService);
	content += getContentEnd();
	Document response = null;
	try {
	    response = sendData(content, webService);
	} catch (ParserConfigurationException ex) {
	    Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
	} catch (SAXException ex) {
	    Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
	}

	insertSmsResponse ret = parseInsertSmsResponse(response);

	return ret;
    }
    /**
     * Send 16bit coded SMS with timing.
     * @param userName Username of the user.
     * @param password Password of the user.
     * @param text The text of the SMS.
     * @param recipientList List of string containing the phone number of recipients.
     * @param wantedSenderPhones List of dedicated modems to use for sending the SMS.
     * @param sendingDate The date when the SMS should be sent.
     * @return insertSmsOutNormal: Integer[] retrunValue - the ID of the SMS
     * @throws IOException
     */
    public static insertSmsResponse insertSmsOutNormal16BitWithTiming(String userName, String password, String text, String[] recipientList, String[] wantedSenderPhones, java.util.Date sendingDate) throws IOException
    {
	return insertSmsOutNormal16BitWithTiming(new SMSUser(userName, password), text, recipientList, wantedSenderPhones, sendingDate);
    }
    /**
     * Send 16bit coded SMS with timing.
     * @param user SMSUser (username,password).
     * @param text The text of the SMS.
     * @param recipientList List of string containing the phone number of recipients.
     * @param wantedSenderPhones List of dedicated modems to use for sending the SMS.
     * @param sendingDate The date when the SMS should be sent.
     * @return insertSmsOutNormal: Integer[] retrunValue - the ID of the SMS
     * @throws IOException
     */
    public static insertSmsResponse insertSmsOutNormal16BitWithTiming(SMSUser user, String text, String[] recipientList, String[] wantedSenderPhones, java.util.Date sendingDate) throws IOException
    {
	String webService = "insertSmsOutNormal16BitWithTiming";

	String content = getContentHeader();
	content += getWebServiceHeader(webService);

	//
	content += "<text>" + text + "</text>\r\n";

	DateFormat df = new SimpleDateFormat("yyyy-MM-dd-HH:mm:ss.SSS");
	String sendingdate = df.format(sendingDate);
	char[] data = sendingdate.toCharArray();
	data[10] = 'T';
	sendingdate = new String(data);
	content += "<sendingdate>" + sendingdate + "</sendingdate>\r\n";

	content += "<recipientList>\r\n";
	for (Integer i = 0; i < recipientList.length; i++) {
	    content += "<string>" + recipientList[i] + "</string>\r\n";
	}
	content += "</recipientList>\r\n";

	if (wantedSenderPhones != null) {
	    for (Integer i = 0; i < wantedSenderPhones.length; i++) {
		if (i == 0) {
		    content += "<wantedSenderPhones>\r\n";
		}
		if (wantedSenderPhones[i] != null) {
		    content += "<user>" + wantedSenderPhones[i] + "</user>";
		}
		if (i == wantedSenderPhones.length - 1) {
		    content += "</wantedSenderPhones>\r\n";
		}
	    }
	}

	//
	content += getUserToString(user);

	content += getWebServiceEnd(webService);
	content += getContentEnd();
	Document response = null;
	try {
	    response = sendData(content, webService);
	} catch (ParserConfigurationException ex) {
	    Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
	} catch (SAXException ex) {
	    Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
	}

	insertSmsResponse ret = parseInsertSmsResponse(response);

	return ret;
    }
    //
    // Private föggvények
    //
    private static org.w3c.dom.Document sendData(String content, String webService) throws IOException, ParserConfigurationException, SAXException
    {

	java.net.URL url = new java.net.URL(serverUrl);
	java.net.URLConnection urlConn = url.openConnection();
	java.io.DataOutputStream outputStream;
	java.io.DataInputStream inputStream;

	urlConn.setDoInput(true);
	urlConn.setDoOutput(true);
	urlConn.setUseCaches(false);
	urlConn.setRequestProperty("Content-Type", "text/xml;charset=utf-8");
	//urlConn.setRequestProperty("SOAPAction", "http://temuri.org/"+webService);

	outputStream = new java.io.DataOutputStream(urlConn.getOutputStream());
	outputStream.write(content.getBytes("UTF8"));
	outputStream.flush();
	outputStream.close();

	inputStream = new java.io.DataInputStream(urlConn.getInputStream());

	//byte[] b = new byte[inputStream.available()];
	//inputStream.readFully(b);
	//String reply = new String(b);

	javax.xml.parsers.DocumentBuilderFactory dbf = javax.xml.parsers.DocumentBuilderFactory.newInstance();
	javax.xml.parsers.DocumentBuilder db = dbf.newDocumentBuilder();
	Document doc = db.parse(inputStream);
	doc.getDocumentElement().normalize();
	return doc;
    }
    private static String getContentHeader()
    {
	return "<?xml version=\"1.0\" encoding=\"utf-8\"?>\r\n"
	       + "<soap:Envelope xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\">\r\n"
	       + "<soap:Body>\r\n";
    }
    private static String getContentEnd()
    {
	return "</soap:Body>\r\n"
	       + "</soap:Envelope>";
    }
    private static String getWebServiceHeader(String webService)
    {
	return "<" + webService + " xmlns=\"http://tempuri.org/\">\r\n";
    }
    private static String getWebServiceEnd(String webService)
    {
	return "</" + webService + ">\r\n";
    }
    private static String getUserToString(SMSUser user)
    {
	return "<user>" + user.getUser() + "</user>\r\n"
	       + "<password>" + user.getPassword() + "</password>\r\n";
    }
    private static getCreditResponse pharseGetCreditResponse(Document doc)
    {
	getCreditResponse ret = null;

	boolean hasError = getHasError(doc);
	String errorString = getErrorString(doc);
	String userErrorString = getUserErrorString(doc);

	//get Real value
	NodeList firstList = doc.getElementsByTagName("returnValue");
	Node firstNode = firstList.item(0);
	Node secondNode = firstNode.getFirstChild();
	Integer returnValue = null;
	if (secondNode != null) {
	    returnValue = Integer.parseInt(secondNode.getNodeValue());
	}
	//

	ret = new getCreditResponse(hasError, errorString, userErrorString, returnValue);
	return ret;
    }
    private static boolean getHasError(Document doc)
    {
	NodeList firstList = doc.getElementsByTagName("hasError");
	Node firstNode = firstList.item(0);
	Node secondNode = firstNode.getFirstChild();
	if (secondNode != null) {
	    return Boolean.parseBoolean(secondNode.getNodeValue());
	}
	return false;
    }
    private static String getErrorString(Document doc)
    {
	NodeList firstList = doc.getElementsByTagName("errorString");
	Node firstNode = firstList.item(0);
	if (firstNode == null) {
	    return null;
	}
	Node secondNode = firstNode.getFirstChild();
	if (secondNode != null) {
	    return secondNode.getNodeValue();
	}
	return null;
    }
    private static String getUserErrorString(Document doc)
    {
	NodeList firstList = doc.getElementsByTagName("userErrorString");
	Node firstNode = firstList.item(0);
	if (firstNode == null) {
	    return null;
	}
	Node secondNode = firstNode.getFirstChild();
	if (secondNode != null) {
	    return secondNode.getNodeValue();
	}
	return null;
    }
    private static deleteSmsResponse pharseDeleteSmsResponse(Document doc)
    {
	deleteSmsResponse ret = null;

	boolean hasError = getHasError(doc);
	String errorString = getErrorString(doc);
	String userErrorString = getUserErrorString(doc);

	//get Real value

	//

	ret = new deleteSmsResponse(hasError, errorString, userErrorString);
	return ret;
    }
    private static getModemsByAccountResponse pharseGetModemsByAccountResponse(Document doc)
    {
	getModemsByAccountResponse ret = null;

	boolean hasError = getHasError(doc);
	String errorString = getErrorString(doc);
	String userErrorString = getUserErrorString(doc);

	//get Real values
	NodeList firstList = doc.getElementsByTagName("returnValueToParse");
	Node firstNode = firstList.item(0);
	Node secondNode = firstNode.getFirstChild();
	String returnValueToParse = "";
	if (secondNode != null) {
	    returnValueToParse = secondNode.getNodeValue();
	}
	String[] returnValue = null;
	firstList = doc.getElementsByTagName("string");
	if (firstList.getLength() > 0) {
	    returnValue = new String[firstList.getLength()];
	    for (Integer i = 0; i < firstList.getLength(); i++) {
		firstNode = firstList.item(i);
		secondNode = firstNode.getFirstChild();
		if (secondNode != null) {
		    returnValue[i] = secondNode.getNodeValue();
		    continue;
		}
		returnValue[i] = null;
	    }
	}
	//

	ret = new getModemsByAccountResponse(hasError, errorString, userErrorString, returnValue, returnValueToParse);
	return ret;
    }
    private static getSmsInResponse parseGetSmsInResponse(Document doc)
    {
	getSmsInResponse ret = null;

	boolean hasError = getHasError(doc);
	String errorString = getErrorString(doc);
	String userErrorString = getUserErrorString(doc);

	// <editor-fold desc="Parsing uniqe return values">
	Element firstNode = null;
	Node secondNode = null;
	Node thirdNode = null;
	NodeList secondList = null;
	NodeList firstList = doc.getElementsByTagName("SMSIn");
	SMSIn[] returnValue = null;
	if (firstList.getLength() > 0) {
	    returnValue = new SMSIn[firstList.getLength()];
	    for (Integer i = 0; i < firstList.getLength(); i++) {
		// <editor-fold defaultstate="collapsed" desc="SMSIn declaration">
		Integer id = null;
		String text = null;
		String sender_phone_number = null;
		String recipient_phone_number = null;
		Date delivery_time = null;
		String calculated_recipient_user = null;
		Integer calculated_recipient_account = null;
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseId">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("id");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			id = Integer.parseInt(thirdNode.getNodeValue());
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseText">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("text");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			text = thirdNode.getNodeValue();
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseSender_phone_number">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("sender_phone_number");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			sender_phone_number = thirdNode.getNodeValue();
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseRecipient_phone_number">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("recipient_phone_number");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			recipient_phone_number = thirdNode.getNodeValue();
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="delivery_time">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("delivery_time");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			DateFormat df = new SimpleDateFormat("yyyy-MM-dd-HH:mm:ss.SSS");//"2010-11-03T07:40:20.507"
			String stringDate = thirdNode.getNodeValue();
			stringDate = stringDate.replace("T", "-");
			try {
			    if (!stringDate.contains(".")) {
				stringDate += ".000";
			    }
			    delivery_time = df.parse(stringDate);
			} catch (ParseException ex) {
			    Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
			}
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseCalculated_recipient_user">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("calculated_recipient_user");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			calculated_recipient_user = thirdNode.getNodeValue();
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseCalculated_recipient_account">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("calculated_recipient_account");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			calculated_recipient_account = Integer.parseInt(thirdNode.getNodeValue());
		    }
		}
		// </editor-fold>

		returnValue[i] = new SMSIn(id, text, sender_phone_number, recipient_phone_number, delivery_time, calculated_recipient_user, calculated_recipient_account);
	    }
	}
	// // </editor-fold>

	ret = new getSmsInResponse(hasError, errorString, userErrorString, returnValue);

	return ret;
    }
    private static getSmsOutResponse parseGetSmsOutResponse(Document doc)
    {
	getSmsOutResponse ret = null;

	boolean hasError = getHasError(doc);
	String errorString = getErrorString(doc);
	String userErrorString = getUserErrorString(doc);

	// <editor-fold desc="Parsing uniqe return values">
	Element firstNode = null;
	Node secondNode = null;
	Node thirdNode = null;
	NodeList secondList = null;
	NodeList firstList = doc.getElementsByTagName("SMSOut");
	SMSOut[] returnValue = null;
	if (firstList.getLength() > 0) {
	    returnValue = new SMSOut[firstList.getLength()];
	    for (Integer i = 0; i < firstList.getLength(); i++) {
		// <editor-fold defaultstate="collapsed" desc="SMSOut declaration">
		wapInfoClass wapInfo = null;
		String[] selectedModemsToSend = null;
		Boolean concatenated = null;
		String wantedCom = null;
		String recipientProvider = null;
		String Text = null;
		Integer id = null;
		Date tryingDate = null;
		Boolean repeatedSending = null;
		String coding = null;
		Boolean flash = null;
		Boolean ringTone = null;
		Boolean wapPush = null;
		Boolean dedicated = null;
		Date wantedSendingTime = null;
		String usedSenderPhoneNumber = null;
		String senderPhoneNumber = null;
		String recipientPhoneNumber = null;
		Short part_count = null;
		Integer account = null;
		String user = null;
		String status = null;
		Date smsSendedTime = null;
		Date smsStatusReportRecieved = null;
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseWapInfo">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("wapInfo");
		secondList.getLength();
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    // <editor-fold defaultstate="collapsed" desc="WapInfo declaration">
		    Element wap = (Element) secondNode;
		    String type = null;
		    String text = null;
		    String link = null;
		    String href = null;
		    String mode = null;
		    Date validityTime = null;
		    Date releaseTime = null;
		    // </editor-fold>
		    // <editor-fold defaultstate="collapsed" desc="type">
		    secondList = wap.getElementsByTagName("type");
		    secondNode = secondList.item(0);
		    if (secondNode != null) {
			thirdNode = secondNode.getFirstChild();
			if (thirdNode != null) {
			    type = thirdNode.getNodeValue();
			}
		    }
		    // </editor-fold>
		    // <editor-fold defaultstate="collapsed" desc="text">
		    secondList = wap.getElementsByTagName("text");
		    secondNode = secondList.item(0);
		    if (secondNode != null) {
			thirdNode = secondNode.getFirstChild();
			if (thirdNode != null) {
			    text = thirdNode.getNodeValue();
			}
		    }
		    // </editor-fold>
		    // <editor-fold defaultstate="collapsed" desc="link">
		    secondList = wap.getElementsByTagName("link");
		    secondNode = secondList.item(0);
		    if (secondNode != null) {
			thirdNode = secondNode.getFirstChild();
			if (thirdNode != null) {
			    link = thirdNode.getNodeValue();
			}
		    }
		    // </editor-fold>
		    // <editor-fold defaultstate="collapsed" desc="href">
		    secondList = wap.getElementsByTagName("href");
		    secondNode = secondList.item(0);
		    if (secondNode != null) {
			thirdNode = secondNode.getFirstChild();
			if (thirdNode != null) {
			    href = thirdNode.getNodeValue();
			}
		    }
		    // </editor-fold>
		    // <editor-fold defaultstate="collapsed" desc="mode">
		    secondList = wap.getElementsByTagName("mode");
		    secondNode = secondList.item(0);
		    if (secondNode != null) {
			thirdNode = secondNode.getFirstChild();
			if (thirdNode != null) {
			    mode = thirdNode.getNodeValue();
			}
		    }
		    // </editor-fold>
		    // <editor-fold defaultstate="collapsed" desc="validityTime">
		    secondList = firstNode.getElementsByTagName("validityTime");
		    secondNode = secondList.item(0);
		    if (secondNode != null) {
			thirdNode = secondNode.getFirstChild();
			if (thirdNode != null) {
			    DateFormat df = new SimpleDateFormat("yyyy-MM-dd-HH:mm:ss.SSS");//"2010-11-03T07:40:20.507"
			    String stringDate = thirdNode.getNodeValue();
			    stringDate = stringDate.replace("T", "-");
			    try {
				if (!stringDate.contains(".")) {
				    stringDate += ".000";
				}
				validityTime = df.parse(stringDate);
			    } catch (ParseException ex) {
				Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
			    }
			}
		    }
		    // </editor-fold>
		    // <editor-fold defaultstate="collapsed" desc="releaseTime">
		    secondList = firstNode.getElementsByTagName("releaseTime");
		    secondNode = secondList.item(0);
		    if (secondNode != null) {
			thirdNode = secondNode.getFirstChild();
			if (thirdNode != null) {
			    DateFormat df = new SimpleDateFormat("yyyy-MM-dd-HH:mm:ss.SSS");//"2010-11-03T07:40:20.507"
			    String stringDate = thirdNode.getNodeValue();
			    stringDate = stringDate.replace("T", "-");
			    try {
				if (!stringDate.contains(".")) {
				    stringDate += ".000";
				}
				releaseTime = df.parse(stringDate);
			    } catch (ParseException ex) {
				Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
			    }
			}
		    }
		    // </editor-fold>
		    wapInfo = new wapInfoClass(type, text, link, href, mode, validityTime, releaseTime);
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseSelectedModemsToSend">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("String");

		selectedModemsToSend = new String[secondList.getLength()];
		for (Integer j = 0; j < secondList.getLength(); j++) {
		    secondNode = secondList.item(i);
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			selectedModemsToSend[i] = thirdNode.getNodeValue();
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseConcatenated">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("concatenated");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			concatenated = Boolean.parseBoolean(thirdNode.getNodeValue());
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseWantedCom">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("wantedCom");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			wantedCom = thirdNode.getNodeValue();
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseRecipientProvider">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("recipientProvider");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			recipientProvider = thirdNode.getNodeValue();
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseText">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("Text");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			Text = thirdNode.getNodeValue();
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseId">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("id");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			id = Integer.parseInt(thirdNode.getNodeValue());
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseTryingDate">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("tryingDate");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    DateFormat df = new SimpleDateFormat("yyyy-MM-dd-HH:mm:ss.SSS");//"2010-11-03T07:40:20.507"
		    String stringDate = secondNode.getFirstChild().getNodeValue();
		    stringDate = stringDate.replace("T", "-");
		    try {
			if (!stringDate.contains(".")) {
			    stringDate += ".000";
			}
			tryingDate = df.parse(stringDate);
		    } catch (ParseException ex) {
			Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
		    }

		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseRepeatedSending">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("repeatedSending");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			repeatedSending = Boolean.parseBoolean(thirdNode.getNodeValue());
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseCoding">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("coding");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			coding = thirdNode.getNodeValue();
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseFlash">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("flash");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			flash = Boolean.parseBoolean(thirdNode.getNodeValue());
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseRingTone">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("ringTone");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			ringTone = Boolean.parseBoolean(thirdNode.getNodeValue());
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseWapPush">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("wapPush");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			wapPush = Boolean.parseBoolean(thirdNode.getNodeValue());
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseDedicated">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("dedicated");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			dedicated = Boolean.parseBoolean(thirdNode.getNodeValue());
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseWantedSendingTime">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("wantedSendingTime");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    DateFormat df = new SimpleDateFormat("yyyy-MM-dd-HH:mm:ss.SSS");//"2010-11-03T07:40:20.507"
		    String stringDate = secondNode.getFirstChild().getNodeValue();
		    stringDate = stringDate.replace("T", "-");
		    try {
			if (!stringDate.contains(".")) {
			    stringDate += ".000";
			}
			wantedSendingTime = df.parse(stringDate);
		    } catch (ParseException ex) {
			Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
		    }

		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseUsedSenderPhoneNumber">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("usedSenderPhoneNumber");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			usedSenderPhoneNumber = thirdNode.getNodeValue();
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseSenderPhoneNumber">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("senderPhoneNumber");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			senderPhoneNumber = thirdNode.getNodeValue();
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseRecipientPhoneNumber">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("recipientPhoneNumber");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			recipientPhoneNumber = thirdNode.getNodeValue();
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parsePart_count">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("part_count");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			part_count = Short.parseShort(thirdNode.getNodeValue());
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseAccount">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("account");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			account = Integer.parseInt(thirdNode.getNodeValue());
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseUser">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("user");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			user = thirdNode.getNodeValue();
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseStatus">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("status");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			status = thirdNode.getNodeValue();
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseSmsSendedTime">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("smsSendedTime");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    DateFormat df = new SimpleDateFormat("yyyy-MM-dd-HH:mm:ss.SSS");//"2010-11-03T07:40:20.507"
		    String stringDate = secondNode.getFirstChild().getNodeValue();
		    stringDate = stringDate.replace("T", "-");
		    try {
			if (!stringDate.contains(".")) {
			    stringDate += ".000";
			}
			smsSendedTime = df.parse(stringDate);
		    } catch (ParseException ex) {
			Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
		    }

		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseSmsStatusReportRecieved">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("smsStatusReportRecieved");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    DateFormat df = new SimpleDateFormat("yyyy-MM-dd-HH:mm:ss.SSS");//"2010-11-03T07:40:20.507"
		    String stringDate = secondNode.getFirstChild().getNodeValue();
		    stringDate = stringDate.replace("T", "-");
		    try {
			if (!stringDate.contains(".")) {
			    stringDate += ".000";
			}
			smsStatusReportRecieved = df.parse(stringDate);
		    } catch (ParseException ex) {
			Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
		    }

		}
		// </editor-fold>

		returnValue[i] = new SMSOut(wapInfo, selectedModemsToSend, concatenated, wantedCom, recipientProvider, Text, id, tryingDate, repeatedSending, coding, flash, ringTone, wapPush, dedicated, wantedSendingTime, usedSenderPhoneNumber, senderPhoneNumber, recipientPhoneNumber, part_count, account, user, status, smsSendedTime, smsStatusReportRecieved);
	    }
	}
	// </editor-fold>

	ret = new getSmsOutResponse(hasError, errorString, userErrorString, returnValue);
	return ret;

    }
    private static getSmsOutResponse parseGetSmsOutResponseId(Document doc)
    {
	getSmsOutResponse ret = null;

	boolean hasError = getHasError(doc);
	String errorString = getErrorString(doc);
	String userErrorString = getUserErrorString(doc);

	// <editor-fold desc="Parsing uniqe return values">
	Element firstNode = null;
	Node secondNode = null;
	Node thirdNode = null;
	NodeList secondList = null;
	NodeList firstList = doc.getElementsByTagName("sms");
	SMSOut[] returnValue = null;
	if (firstList.getLength() > 0) {
	    returnValue = new SMSOut[firstList.getLength()];
	    for (Integer i = 0; i < firstList.getLength(); i++) {
		// <editor-fold defaultstate="collapsed" desc="SMSOut declaration">
		wapInfoClass wapInfo = null;
		String[] selectedModemsToSend = null;
		Boolean concatenated = null;
		String wantedCom = null;
		String recipientProvider = null;
		String text = null;
		Integer id = null;
		Date tryingDate = null;
		Boolean repeatedSending = null;
		String coding = null;
		Boolean flash = null;
		Boolean ringTone = null;
		Boolean wapPush = null;
		Boolean dedicated = null;
		Date wantedSendingTime = null;
		String usedSenderPhoneNumber = null;
		String senderPhoneNumber = null;
		String recipientPhoneNumber = null;
		Short part_count = null;
		Integer account = null;
		String user = null;
		String status = null;
		Date smsSendedTime = null;
		Date smsStatusReportRecieved = null;
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseWapInfo">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("wapInfo");
		secondList.getLength();
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    // <editor-fold defaultstate="collapsed" desc="WapInfo declaration">
		    Element wap = (Element) secondNode;
		    String type = null;
		    String text_ = null;
		    String link = null;
		    String href = null;
		    String mode = null;
		    Date validityTime = null;
		    Date releaseTime = null;
		    // </editor-fold>
		    // <editor-fold defaultstate="collapsed" desc="type">
		    secondList = wap.getElementsByTagName("type");
		    secondNode = secondList.item(0);
		    if (secondNode != null) {
			thirdNode = secondNode.getFirstChild();
			if (thirdNode != null) {
			    type = thirdNode.getNodeValue();
			}
		    }
		    // </editor-fold>
		    // <editor-fold defaultstate="collapsed" desc="text">
		    secondList = wap.getElementsByTagName("text");
		    secondNode = secondList.item(0);
		    if (secondNode != null) {
			thirdNode = secondNode.getFirstChild();
			if (thirdNode != null) {
			    text = thirdNode.getNodeValue();
			}
		    }
		    // </editor-fold>
		    // <editor-fold defaultstate="collapsed" desc="link">
		    secondList = wap.getElementsByTagName("link");
		    secondNode = secondList.item(0);
		    if (secondNode != null) {
			thirdNode = secondNode.getFirstChild();
			if (thirdNode != null) {
			    link = thirdNode.getNodeValue();
			}
		    }
		    // </editor-fold>
		    // <editor-fold defaultstate="collapsed" desc="href">
		    secondList = wap.getElementsByTagName("href");
		    secondNode = secondList.item(0);
		    if (secondNode != null) {
			thirdNode = secondNode.getFirstChild();
			if (thirdNode != null) {
			    href = thirdNode.getNodeValue();
			}
		    }
		    // </editor-fold>
		    // <editor-fold defaultstate="collapsed" desc="mode">
		    secondList = wap.getElementsByTagName("mode");
		    secondNode = secondList.item(0);
		    if (secondNode != null) {
			thirdNode = secondNode.getFirstChild();
			if (thirdNode != null) {
			    mode = thirdNode.getNodeValue();
			}
		    }
		    // </editor-fold>
		    // <editor-fold defaultstate="collapsed" desc="validityTime">
		    secondList = firstNode.getElementsByTagName("validityTime");
		    secondNode = secondList.item(0);
		    if (secondNode != null) {
			thirdNode = secondNode.getFirstChild();
			if (thirdNode != null) {
			    DateFormat df = new SimpleDateFormat("yyyy-MM-dd-HH:mm:ss.SSS");//"2010-11-03T07:40:20.507"
			    String stringDate = thirdNode.getNodeValue();
			    stringDate = stringDate.replace("T", "-");
			    try {
				if (!stringDate.contains(".")) {
				    stringDate += ".000";
				}
				validityTime = df.parse(stringDate);
			    } catch (ParseException ex) {
				Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
			    }
			}
		    }
		    // </editor-fold>
		    // <editor-fold defaultstate="collapsed" desc="releaseTime">
		    secondList = firstNode.getElementsByTagName("releaseTime");
		    secondNode = secondList.item(0);
		    if (secondNode != null) {
			thirdNode = secondNode.getFirstChild();
			if (thirdNode != null) {
			    DateFormat df = new SimpleDateFormat("yyyy-MM-dd-HH:mm:ss.SSS");//"2010-11-03T07:40:20.507"
			    String stringDate = thirdNode.getNodeValue();
			    stringDate = stringDate.replace("T", "-");
			    try {
				if (!stringDate.contains(".")) {
				    stringDate += ".000";
				}
				releaseTime = df.parse(stringDate);
			    } catch (ParseException ex) {
				Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
			    }
			}
		    }
		    // </editor-fold>
		    wapInfo = new wapInfoClass(type, text_, link, href, mode, validityTime, releaseTime);
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseSelectedModemsToSend">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("string");

		selectedModemsToSend = new String[secondList.getLength()];
		for (Integer j = 0; j < secondList.getLength(); j++) {
		    secondNode = secondList.item(i);
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			selectedModemsToSend[i] = thirdNode.getNodeValue();
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseConcatenated">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("concatenated");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			concatenated = Boolean.parseBoolean(thirdNode.getNodeValue());
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseWantedCom">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("wantedCom");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			wantedCom = thirdNode.getNodeValue();
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseRecipientProvider">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("recipientProvider");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			recipientProvider = thirdNode.getNodeValue();
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseText">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("text");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			text = thirdNode.getNodeValue();
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseId">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("id");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			id = Integer.parseInt(thirdNode.getNodeValue());
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseTryingDate">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("tryingDate");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    DateFormat df = new SimpleDateFormat("yyyy-MM-dd-HH:mm:ss.SSS");//"2010-11-03T07:40:20.507"
		    String stringDate = secondNode.getFirstChild().getNodeValue();
		    stringDate = stringDate.replace("T", "-");
		    try {
			if (!stringDate.contains(".")) {
			    stringDate += ".000";
			}
			tryingDate = df.parse(stringDate);
		    } catch (ParseException ex) {
			Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
		    }

		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseRepeatedSending">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("repeatedSending");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			repeatedSending = Boolean.parseBoolean(thirdNode.getNodeValue());
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseCoding">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("coding");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			coding = thirdNode.getNodeValue();
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseFlash">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("flash");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			flash = Boolean.parseBoolean(thirdNode.getNodeValue());
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseRingTone">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("ringTone");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			ringTone = Boolean.parseBoolean(thirdNode.getNodeValue());
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseWapPush">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("wapPush");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			wapPush = Boolean.parseBoolean(thirdNode.getNodeValue());
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseDedicated">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("dedicated");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			dedicated = Boolean.parseBoolean(thirdNode.getNodeValue());
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseWantedSendingTime">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("wantedSendingTime");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    DateFormat df = new SimpleDateFormat("yyyy-MM-dd-HH:mm:ss.SSS");//"2010-11-03T07:40:20.507"
		    String stringDate = secondNode.getFirstChild().getNodeValue();
		    stringDate = stringDate.replace("T", "-");
		    try {
			if (!stringDate.contains(".")) {
			    stringDate += ".000";
			}
			wantedSendingTime = df.parse(stringDate);
		    } catch (ParseException ex) {
			Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
		    }

		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseUsedSenderPhoneNumber">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("usedSenderPhoneNumber");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			usedSenderPhoneNumber = thirdNode.getNodeValue();
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseSenderPhoneNumber">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("senderPhoneNumber");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			senderPhoneNumber = thirdNode.getNodeValue();
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseRecipientPhoneNumber">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("recipientPhoneNumber");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			recipientPhoneNumber = thirdNode.getNodeValue();
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parsePart_count">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("part_count");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			part_count = Short.parseShort(thirdNode.getNodeValue());
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseAccount">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("account");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			account = Integer.parseInt(thirdNode.getNodeValue());
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseUser">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("user");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			user = thirdNode.getNodeValue();
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseStatus">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("status");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    thirdNode = secondNode.getFirstChild();
		    if (thirdNode != null) {
			status = thirdNode.getNodeValue();
		    }
		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseSmsSendedTime">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("smsSendedTime");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    DateFormat df = new SimpleDateFormat("yyyy-MM-dd-HH:mm:ss.SSS");//"2010-11-03T07:40:20.507"
		    String stringDate = secondNode.getFirstChild().getNodeValue();
		    stringDate = stringDate.replace("T", "-");
		    try {
			if (!stringDate.contains(".")) {
			    stringDate += ".000";
			}
			smsSendedTime = df.parse(stringDate);
		    } catch (ParseException ex) {
			Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
		    }

		}
		// </editor-fold>
		// <editor-fold defaultstate="collapsed" desc="parseSmsStatusReportRecieved">
		firstNode = (Element) firstList.item(i);
		secondList = firstNode.getElementsByTagName("smsStatusReportRecieved");
		secondNode = secondList.item(0);
		if (secondNode != null) {
		    DateFormat df = new SimpleDateFormat("yyyy-MM-dd-HH:mm:ss.SSS");//"2010-11-03T07:40:20.507"
		    String stringDate = secondNode.getFirstChild().getNodeValue();
		    stringDate = stringDate.replace("T", "-");
		    try {
			if (!stringDate.contains(".")) {
			    stringDate += ".000";
			}
			smsStatusReportRecieved = df.parse(stringDate);
		    } catch (ParseException ex) {
			Logger.getLogger(SmsGateway.class.getName()).log(Level.SEVERE, null, ex);
		    }

		}
		// </editor-fold>

		returnValue[i] = new SMSOut(wapInfo, selectedModemsToSend, concatenated, wantedCom, recipientProvider, text, id, tryingDate, repeatedSending, coding, flash, ringTone, wapPush, dedicated, wantedSendingTime, usedSenderPhoneNumber, senderPhoneNumber, recipientPhoneNumber, part_count, account, user, status, smsSendedTime, smsStatusReportRecieved);
	    }
	}
	// </editor-fold>

	ret = new getSmsOutResponse(hasError, errorString, userErrorString, returnValue);
	return ret;
    }
    private static insertSmsResponse parseInsertSmsResponse(Document doc)
    {
	insertSmsResponse ret = null;

	boolean hasError = getHasError(doc);
	String errorString = getErrorString(doc);
	String userErrorString = getUserErrorString(doc);

	//get Real values
	NodeList firstList;
	Node firstNode;
	Node secondNode;

	firstList = doc.getElementsByTagName("int");
	Integer[] returnValue = null;
	if (firstList.getLength() > 0) {
	    returnValue = new Integer[firstList.getLength()];
	    for (Integer i = 0; i < firstList.getLength(); i++) {
		firstNode = firstList.item(i);
		secondNode = firstNode.getFirstChild();
		if (secondNode != null) {
		    returnValue[i] = Integer.parseInt(secondNode.getNodeValue());
		}
	    }
	}
	//

	ret = new insertSmsResponse(hasError, errorString, userErrorString, returnValue);
	return ret;

    }
    private static getSmsLengthResponse pharseGetSmsLengthResponse(Document doc)
    {
	getSmsLengthResponse ret = null;

	boolean hasError = getHasError(doc);
	String errorString = getErrorString(doc);
	String userErrorString = getUserErrorString(doc);

	//get Real value
	NodeList firstList = doc.getElementsByTagName("returnValue");
	Node firstNode = firstList.item(0);
	Node secondNode = firstNode.getFirstChild();
	Integer returnValue = null;
	if (secondNode != null) {
	    Integer.parseInt(secondNode.getNodeValue());
	}
	//

	ret = new getSmsLengthResponse(hasError, errorString, userErrorString, returnValue);
	return ret;
    }
}
